/*------------------------------------------------------------------------------
*
* Copyright 2001-2007 Dialogic(R) Corporation
*
* All Rights Reserved
*
* This software is the property of Dialogic Corporation and/or its
* subsidiaries ("Dialogic"). This copyright notice may not be removed,
* modified or obliterated without the prior written permission of
* Dialogic.
*
* No right, title, ownership or other interest in the software is hereby
* granted or transferred. The information contained herein is subject
* to change without notice and should not be construed as a commitment of
* Dialogic.
*
* The code is provided by Dialogic "AS IS", without warranty of any
* kind, either expressed or implied. Dialogic shall not be liable for
* any damages arising out of your use of this application, even if it has
* been advised of the possibility of such damages.
*
*------------------------------------------------------------------------------
 *
 * File name:   smssdk.h
 * Project:     Dialogic(R) Diva(R) SDK
 * Description: Public header file for the SMS functionality of the Diva API.
 *
 * Version:     %%InternalVersion%%
 *
------------------------------------------------------------------------------*/

#ifndef _SMSSDK_H_
#define _SMSSDK_H_

#include <dssdk.h>


///////////////////////////////////////////////////////////////////////////////
// SMS DEFINES
///////////////////////////////////////////////////////////////////////////////


#define SMS1_MAX_NUMBER_OF_EVENTS   300
#define SMS1_MAX_NUMBER_OF_CALLS    (16*30) // 16 PRI cards * 30 calls per PRI

// how many digits allowed (GSM 03.40/9.1.2.5)
#define SMS1_MAX_ADDRESS_FIELD      20

// The maximum message length of a Transfer Layer protocol message is 176 octets
// (SMS_STATUS_REPORT) (ES 201 912/5.3.3.1). This maximum is enforced by layer 3.
// Note, however, that other service centers involved in the delivery of the
// message may not allow a frame of this size. Users are advised to keep to the
// maximum guaranteed limits as specified in GSM 03.40/9.2.2.x, or they will
// risk a missed frame.
#define SMS1_MAX_L3_PDU_SIZE        176

// For layer 2 messages three octets need to to be added for message type,
// length and checksum.
#define SMS1_MAX_L2_FRAME_SIZE      (SMS1_MAX_L3_PDU_SIZE + 3)

// This is an arbitrary number, that can accomodate the biggest
// user data of all PDUs (159 octets for a positive deliver report), encoded
// in 7 bits (159 * 8 / 7 = 181). It has 3 bytes to spare and has the
// advantage of being a round number, as 184 7-bit characters can be encoded
// in 161 octets. (GSM 03.40/9.2.2.1a)
#define SMS1_MAX_DATA_LENGTH        184


///////////////////////////////////////////////////////////////////////////////
// MESSAGE BUILDING BLOCKS
///////////////////////////////////////////////////////////////////////////////


// Errors are reported using 32 bits. The lower octet is as specified below.
// The remaining three octets are usually zero, but may have non-zero
// values for some errors, to communicate extra information.
typedef enum
{
    // Errors defined here must be between 1 and 255 (0 is success).

    SMS_ERROR_SUCCESS,                      // success

    // utils errors
    SMS1_ERROR_UT_THREAD_CREATE_FAIL,       // thread creation failed
    SMS1_ERROR_UT_INV_NO_EVENTS,            // invalid number of events
    SMS1_ERROR_UT_INIT_FAIL,                // utils init failed

    // general errors
    SMS1_ERROR_GEN_INV_CALL_HANDLE = 0x10,  // invalid call handle - call not created

    // layer 2 - general
    SMS1_ERROR_L2_NOT_RUNNING = 0x20,       // layer 2 not running
    SMS1_ERROR_L2_OUTPUT_OVERFLOW,          // overflow - message received from layer 3 discarded
    SMS1_ERROR_L2_L3_INV_MSG_TYPE,          // invalid type - message received from layer 3 discarded
    SMS1_ERROR_L2_CANT_CREATE_CALL,         // cannot create call at L2 - out of memory
    SMS1_ERROR_L2_FRAME_RX_L1_TOO_BIG,      // frame received from layer 1 too big - discarded

    // layer 3 - general
    SMS1_ERROR_L3_NOT_RUNNING = 0x40,       // layer 3 not running
    SMS1_ERROR_L3_CANT_CREATE_CALL,         // cannot create call at L3 - out of memory

    // layer 3 - receive PDUs
    SMS1_ERROR_L3_RX_MESSAGE_OVERRUN = 0x50,// message overrun
    SMS1_ERROR_L3_RX_TP_VPF_NOT_SUPPORT,    // TP-VPF not supported
    SMS1_ERROR_L3_RX_UNSUP_TYPE_NUMBER,     // unsupported type of number, address not decoded
    SMS1_ERROR_L3_RX_INV_TYPE_ADD,          // invalid type of address, bit 7 not set, address not decoded
    SMS1_ERROR_L3_RX_INV_ADD_LENGTH,        // invalid address length, address not decoded
    SMS1_ERROR_L3_RX_L2_MISS_OCTETS,        // layer 2 frame is missing octets
    SMS1_ERROR_L3_RX_PDU_TOO_BIG,           // received PDU too big - message truncated
    SMS1_ERROR_L3_RX_INV_MTI,               // invalid MTI
    SMS1_ERROR_L3_RX_INV_USER_DATA_HEADER,  // invalid user data header - ignored

    // layer 3 - transmit PDUs
    SMS1_ERROR_L3_TX_INV_TYPE_NUMBER = 0x70,// invalid type of number, PDU not sent
    SMS1_ERROR_L3_TX_INV_DCS,               // invalid data coding scheme
    SMS1_ERROR_L3_TX_L3_INV_APP_MSG_TYPE,   // invalid application message type
    SMS1_ERROR_L3_TX_L3_INV_PDU_TYPE,       // invalid PDU type
    SMS1_ERROR_L3_TX_PDU_TOO_BIG,           // PDU bigger than 176 octets - not sent
} sms_error_t;

typedef enum    // application message type, inspired by ES 201 912/Annex A
{
    SMS1_APP_MSG_UNUSED,

    //--------------------------------------------
    // MESSAGES SENT FROM LAYER 3 TO THE APPLICATION:

    // carries a received PDU (all types)
    SMS1_APP_SMS_IND,

    // Indicates that the SMS connection was released by the remote side, or
    // released after a request by the application. It carries no PDU. When the
    // application receives this message it should (in general) disconnect the
    // call.
    SMS1_CM_REL_IND,

    // Reports one of the following SMS data link errors:
    //   1. a SMS1_DLL_SMS_ERROR frame was received twice
    //   2. no data link establish message received
    //   3. PDU sent, but no reply was received in the configured period
    //
    // The type of error is reported in tl_data_rej_cause in sms1_app_message_t.
    // When the application receives this message it should send SMS1_TL_SMS_REL
    // (see a few lines down) to layer 3, to initiate breaking-up of the SMS
    // connection.
    SMS1_APP_SMS_REJ,

    // Reports that a bad frame was received by our SMS data link. The type of
    // error is reported in tl_data_rej_cause in sms1_app_message_t.
    // When the application receives this message it should disconnect the call.
    SMS1_APP_DL_ERROR,

    //--------------------------------------------
    // MESSAGES SENT FROM THE APPLICATION TO LAYER 3:

    // carries PDU to be sent to remote party (all types)
    SMS1_TL_SMS_IND,

    // requests that SMS connection be established, carries no PDU
    SMS1_TL_EST_IND,

    // requests that SMS connection be released, carries no PDU
    SMS1_TL_SMS_REL,
} sms1_app_msg_type_t;

// transfer layer messages - GSM 03.40/9.2.2
typedef enum
{
    SMS1_PDU_TYPE_NOT_USED,

    // from service center to mobile station
    SMS1_PDU_TYPE_DELIVER,          // conveys short message
    SMS1_PDU_TYPE_STATUS_REPORT,    // conveys status report
    SMS1_PDU_TYPE_POS_SUBMIT_REP,   // conveys positive submit report
    SMS1_PDU_TYPE_NEG_SUBMIT_REP,   // conveys negative submit report

    // from mobile station to service center
    SMS1_PDU_TYPE_SUBMIT,           // conveys short message
    SMS1_PDU_TYPE_COMMAND,          // conveys command
    SMS1_PDU_TYPE_POS_DELIVER_REP,  // conveys positive deliver report
    SMS1_PDU_TYPE_NEG_DELIVER_REP,  // conveys negative deliver report
} sms1_pdu_type_t;

// data reject cause - indicates an error at data link level
typedef enum
{
    SMS1_TL_DATA_REJ_CAUSE_UNUSED,

    //--------------------------------------------------------------------
    // THE FOLLOWING TWO ERRORS ARE USED WITH MESSAGE TYPE SMS1_APP_SMS_REJ:

    // no message received from other side after call established by us
    SMS1_TL_DATA_REJ_CAUSE_TIMEOUT_NO_SIGN_LIFE,

    // no acknowledgment received to sent PDU
    SMS1_TL_DATA_REJ_CAUSE_TIMEOUT_NO_ACK,

    //--------------------------------------------------------------------
    // THE FOLLOWING FIVE ERRORS ARE USED WITH MESSAGE TYPE SMS1_APP_SMS_REJ
    // AND SMS1_APP_DL_ERR.


    // message had wrong checksum
    SMS1_TL_DATA_REJ_CAUSE_CHECKKSUM,

    // message had wrong message length
    SMS1_TL_DATA_REJ_CAUSE_LENGTH,

    // message had wrong type
    SMS1_TL_DATA_REJ_CAUSE_TYPE,

    // message used extension mechanism (not supported)
    SMS1_TL_DATA_REJ_CAUSE_EXT_NOT_SUP,

    // message had unspecified error
    SMS1_TL_DATA_REJ_CAUSE_UNSPEC,

} sms1_tl_data_rej_cause_t;

typedef struct
{
    BYTE year;          // Only the last two digits of the year!
    BYTE month;
    BYTE day;
    BYTE hour;
    BYTE minute;
    BYTE second;

    // gmt_offset indicates the difference between the local tine and GMT in minutes
    int             gmt_offset;
} sms1_time_t;

typedef enum
{
    // Short message transaction completed

    SMS1_STAT_SM_RECEIVED = 0x00,       // short message received by the SME
    SMS1_STAT_SM_FORWARDED,             // short message forwarded by the SC to
                                        // the SME but the SC is unable
                                        // to confirm delivery
    SMS1_STAT_SM_REPLACED,              // short message replaced by the SC

    // Temporary error, SC still trying to transfer SM.
    // TT stands for "temporary, trying".

    SMS1_STAT_TT_CONGESTION = 0x20,     // congestion
    SMS1_STAT_TT_SME_BUSY,              // SME busy
    SMS1_STAT_TT_NO_RESPONSE_FROM_SME,  // no response from SME
    SMS1_STAT_TT_SERVICE_REJECTED,      // service rejected
    SMS1_STAT_TT_QUAL_OF_SERV_NOT_AVAIL,// quality of service not available
    SMS1_STAT_TT_ERROR_IN_SME,          // error in SME

    // Permanent error, SC does not make any more transfer attempts.
    // PE stands for "permanent error".

    SMS1_STAT_PE_REMOTE_PROC_ERROR= 0x40,// remote procedure error
    SMS1_STAT_PE_INCOMPAT_DESTINATION,   // incompatible destination
    SMS1_STAT_PE_CONNECT_REJECTED_BY_SME,// connection rejected by SME
    SMS1_STAT_PE_NOT_OBTAINABLE,         // not obtainable
    SMS1_STAT_PE_QUAL_OF_SERV_NOT_AVAIL, // quality of service not available
    SMS1_STAT_PE_NO_INTERWORK_AVAIL,     // no interworking available
    SMS1_STAT_PE_SM_VALID_PERIOD_EXPIRED,// SM validity period expired
    SMS1_STAT_PE_SM_DEL_BY_ORIG_SME,     // SM deleted by originating SME
    SMS1_STAT_PE_SM_DEL_BY_SC_ADMIN,     // SM deleted by SC administration
    SMS1_STAT_PE_SM_DOES_NOT_EXIST,      // SM does not exist (The SM may have
                                         // previously existed in the SC but the
                                         // SC no longer has knowledge of it or
                                         // the SM may never have previously
                                         // existed in the SC)

    // Temporary error, SC does not make any more transfer attempts.
    // TN stands for "temporary, not trying".

    SMS1_STAT_TN_CONGESTION = 0x60,     // congestion
    SMS1_STAT_TN_SME_BUSY,              // SME busy
    SMS1_STAT_TN_NO_RESP_FROM_SME,      // no response from SME
    SMS1_STAT_TN_SERV_REJECTED,         // service rejected
    SMS1_STAT_TN_QUAL_OF_SERV_NOT_AVAIL,// quality of service not available
    SMS1_STAT_TN_ERROR_IN_SME,          // error in SME

} sms1_status_t;

typedef enum
{
    SMS1_SRQ_SUBMIT,
    SMS1_SRQ_COMMAND,
} sms1_srq_t;           // status report qualifier,

typedef enum
{
    SMS1_FCS_SUCESS,                    // Dummy error in case of no error!

    // 80 - 8F TP-PID errors
    SMS1_FCS_TELE_INTER_NOT_SUP = 0x80, // telematic interworking not supported
    SMS1_FCS_SM_TYP_0_NOT_SUP = 0x81,   // short message Type 0 not supported
    SMS1_FCS_CANT_REPLACE_SM = 0x82,    // cannot replace short message
    SMS1_FCS_UNSPEC_TP_PID_ER = 0x8F,   // unspecified TP-PID error

    // 90 - 9F TP-DCS errors
    SMS1_FCS_DCS_NOT_SUP = 0x90,        // data coding scheme (alphabet) not supported
    SMS1_FCS_MSG_CLASS_NOT_SUP = 0x91,  // message class not supported
    SMS1_FCS_UNSPEC_TP_DCS_ER = 0x9F,   // unspecified TP-DCS error

    // A0 - AF TP-Command Errors
    SMS1_FCS_CMD_CANT_BE_ACT = 0xA0,    // command cannot be actioned
    SMS1_FCS_CMD_NOT_SUP = 0xA1,        // command unsupported
    SMS1_FCS_UNSPEC_TP_CMd_ER = 0xAF,   // unspecified TP-Command error

    SMS1_FCS_TPDU_NOT_SUP = 0xB0,       // TPDU not supported

    // defined in Belgacom BGC_D_48_0204_30_01_E
    SMS1_FCS_INV_DEST_NUMBER = 0xB3,    // invalid destination number
    SMS1_FCS_INV_DN_NUMBER_FMT = 0xB4,  // invalid destination number (invalid number format)

    // miscellaneous Service Center errors
    SMS1_FCS_SC_BUSY = 0xC0,            // SC busy
    SMS1_FCS_NO_SC_SUBSCRIP = 0xC1,     // no SC subscription
    SMS1_FCS_SC_SYS_FAILURE = 0xC2,     // SC system failure
    SMS1_FCS_INV_SME_ADD = 0xC3,        // invalid SME address
    SMS1_FCS_DEST_SME_BARRED = 0xC4,    // destination SME barred
    SMS1_FCS_SM_REJEC_DUP_SM = 0xC5,    // SM Rejected-Duplicate SM
    SMS1_FCS_TP_VPF_NOT_SUP = 0xC6,     // TP-VPF not supported
    SMS1_FCS_TP_VP_NOT_SUP = 0xC7,      // TP-VP not supported

    // miscellaneous Mobile Station errors
    SMS1_FCS_SIM_SMS_STOR_FULL = 0xD0,  // SIM SMS storage full
    SMS1_FCS_NO_SMS_STOR_CAP_SIM = 0xD1,// no SMS storage capability in SIM
    SMS1_FCS_ERROR_IN_MS = 0xD2,        // error in MS
    SMS1_FCS_MEM_CAP_EXCEED = 0xD3,     // Memory Capacity Exceeded
    SMS1_FCS_SIM_APP_TLKIT_BUSY = 0xD4, // SIM Application Toolkit Busy
    SMS1_FCS_SIM_DATA_DOWN_ER = 0xD5,   // SIM data download error

    // E0 - FE Values specific to an application

    // defined in ES 201 912/5.6
    SMS1_FCS_UNKNOWN_SM_TE = 0xE0,      // unknown SM-TE (SM-TE CLI restricted)
    SMS1_FCS_UDHI_NOT_SUP = 0xE1,       // user Data Header not supported

    // defined in Belgacom BGC_D_48_0204_30_01_E
    SMS1_FCS_SM_SC_PROBLEMS = 0xF0,     // problems with SM Service Centre

    // defined in Deutsche Telekom 1TR140/A.3.1
    SMS1_FCS_CANT_CONNECT_SC = 0xFD,    // failure to establish connection to SM-SC
    SMS1_FCS_CANT_SEND_SM = 0xFE,       // failure during transmission of short message

    SMS1_FCS_UNSPEC_ER_CAUSE = 0xFF,    // unspecified error cause
} sms1_failure_cause_t;

typedef enum
{
    // enquiry relating to previously submitted short message
    SMS1_COMMAND_ENQUIRY_SUBMITTED_SM,

    // cancel Status Report Request relating to previously submitted short message
    SMS1_COMMAND_CANCEL_STAT_REP_REQUEST,

    // delete previously submitted Short Message
    SMS1_COMMAND_DELETE_SUBMITTED_SM,

    // enable Status Report Request relating to previously submitted short message
    SMS1_COMMAND_ENABLE_STAT_REP_REQUEST,

    // E0-FF: Service Center specific
} sms1_command_type_t;

typedef enum
{
    SMS1_TYPE_NUMBER_UNKNOWN,       // mandatory in some countries
    SMS1_TYPE_NUMBER_INTERNATIONAL,
    SMS1_TYPE_NUMBER_NATIONAL,

    // the following types are not currently supported
    SMS1_TYPE_NUMBER_NET_SPECIFIC,
    SMS1_TYPE_NUMBER_SUBSCRIBER,
    SMS1_TYPE_NUMBER_ALPHANUMERIC,
    SMS1_TYPE_NUMBER_ABBREVIATED,
    SMS1_TYPE_NUMBER_RESERVED,
} sms1_type_number_t;

typedef enum
{
    SMS1_NO_PLAN_ID_UNKNOWN = 0x0,  // unknown
    SMS1_NO_PLAN_ID_ISDN = 0x1,     // ISDN/telephone numbering plan (E.164/E.163)
                                    // mandatory in some countries
    SMS1_NO_PLAN_ID_DATA = 0x3,     // data numbering plan (X.121)
    SMS1_NO_PLAN_ID_TELEX = 0x4,    // telex numbering plan
    SMS1_NO_PLAN_ID_NATIONAL = 0x8, // national numbering plan
    SMS1_NO_PLAN_ID_PRIVATE = 0x9,  // private numbering plan
    SMS1_NO_PLAN_ID_ERMES = 0xA,    // ERMES numbering plan (ETSI DE/PS 3 01-3)
} sms1_no_plan_id_t;

typedef struct
{
    sms1_type_number_t  type_number;

    // this field is passed transparently by layer 3 (not checked or processed)
    sms1_no_plan_id_t   no_plan_id;

    // must be a NULL-terminated ASCII string, only decimal digits allowed
    char                value[SMS1_MAX_ADDRESS_FIELD + 1];
} sms1_address_t;

typedef enum
{
    SMS1_PID_DEFAULT,

    // telematic interworking
    SMS1_PID_IMPLICIT = 0x20,       // implicit - device type is specific to this SC,
                                    // or can be concluded on the basis of the address
    SMS1_PID_TELEX,                 // telex (or teletex reduced to telex format)
    SMS1_PID_G3_TELEFAX,            // group 3 telefax
    SMS1_PID_G4_TELEFAX,            // group 4 telefax
    SMS1_PID_VOICE_PHONE,           // voice telephone (i.e. conversion to speech)
    SMS1_PID_ERMES,                 // ERMES (European Radio Messaging System)
    SMS1_PID_NAT_PAGE_SYS,          // national paging system (known to the SC)
    SMS1_PID_VIDEOTEX,              // videotex (T.100/T.101)
    SMS1_PID_TTEX_CAR_UNSPEC,       // teletex, carrier unspecified
    SMS1_PID_TTEX_PSPDN,            // teletex, in PSPDN
    SMS1_PID_TTEX_CSPDN,            // teletex, in CSPDN
    SMS1_PID_TTEX_ANA_PSTN,         // teletex, in analog PSTN
    SMS1_PID_TTEX_DIG_ISDN,         // teletex, in digital ISDN
    SMS1_PID_UCI,                   // UCI (Universal Computer Interface, ETSI DE/PS 3 01-3)

    SMS1_PID_MSG_HAND_FACIL = 0x30, // a message handling facility (known to the SC)
    SMS1_PID_PUB_X400_MSG,          // any public X.400-based message handling system
    SMS1_PID_INTERNET_EMAIL,        // internet electronic mail

    SMS1_PID_GSM_MOB_STAT = 0x3f,   // a GSM mobile station

    // replace short message
    SMS1_PID_SM_TYPE_0 = 0x40,      // short message type 0
    SMS1_PID_REPLACE_SM_TYPE_1,     // replace short message type 1
    SMS1_PID_REPLACE_SM_TYPE_2,     // replace short message type 2
    SMS1_PID_REPLACE_SM_TYPE_3,     // replace short message type 3
    SMS1_PID_REPLACE_SM_TYPE_4,     // replace short message type 4
    SMS1_PID_REPLACE_SM_TYPE_5,     // replace short message type 5
    SMS1_PID_REPLACE_SM_TYPE_6,     // replace short message type 6
    SMS1_PID_REPLACE_SM_TYPE_7,     // replace short message type 7

    // miscellaneous
    SMS1_PID_RTN_CALL_MSG = 0x5f,   // return call message

    SMS1_PID_ME_DATA_DOWN = 0x7d,   // ME data download
    SMS1_PID_ME_DEPERSONAL_SM,      // ME de-personalization short message
    SMS1_PID_SIM_DATA_DOWNLOAD      // SIM data download
} sms1_pid_t;

typedef enum    // data coding scheme class
{
    SMS1_DCS_CLASS_0,
    SMS1_DCS_CLASS_1,
    SMS1_DCS_CLASS_2,
    SMS1_DCS_CLASS_3,
    SMS1_DCS_NO_CLASS,
} sms1_dcs_class_t;

typedef enum    // data coding scheme message waiting indication
{
    SMS1_DCS_MWI_NOT_USED,
    SMS1_DCS_MWI_DISCARD_MESSAGE,
    SMS1_DCS_MWI_STORE_MESSAGE,
} sms1_dcs_mwi_t;

typedef enum    // data coding scheme message waiting indication type
{
    SMS1_DCS_MWI_TYPE_VOICE,    // Voicemail Message Waiting
    SMS1_DCS_MWI_TYPE_FAX,      // Fax Message Waiting
    SMS1_DCS_MWI_TYPE_EMAIL,    // Electronic Mail Message Waiting
    SMS1_DCS_MWI_TYPE_OTHER,    // Other Message Waiting*
} sms1_dcs_mwi_type_t;

typedef enum    // data coding scheme alphabet
{
    SMS1_DCS_ALPHA_7_BIT_DEFAULT,
    SMS1_DCS_ALPHA_8_BIT_USER_DATA,
    SMS1_DCS_ALPHA_16_BIT_UCS2,
} sms1_dcs_alphabet_t;

typedef struct // decoded data coding scheme - see GSM 03.38/4
{
    // Message Waiting Indication (MWI)

    // If MWI is used, compression and classes cannot be used.
    // If MWI is not used, set this field to SMS1_DCS_MWI_NOT_USED.
    sms1_dcs_mwi_t      mwi;
    BOOL                mwi_active; // used only if MWI used
    sms1_dcs_mwi_type_t mwi_type;   // used only if MWI used

    // If MWI Store Message is used, both default 7-bit and UCS2 alphabets
    // can be used.
    // If MWI Discard Message is used, only default 7-bit alphabet can be used.
    sms1_dcs_alphabet_t alphabet;

    // If TRUE user data is compressed according to GSM 03.42.
    // COMPRESSION IS NOT CURRENTLY SUPPORTED BY LAYER 3,
    // therefore user data will be passed transparently
    BOOL                compressed;

    sms1_dcs_class_t    dcs_class;
} sms1_decoded_dcs_t;

typedef struct // transmitted data coding scheme
{
    // This field determines which of the following two fields should be
    // used by layer 3. The reason for making available both decoded and
    // "raw" dcs, is that, for some types of coding there is more than one way
    // to encode DCS, and some applications may prefer to keep control of DCS.
    // If you don't want to bother with coding DCS, use the decoded version,
    // and layer 3 will encode it automatically for you.
    BOOL                use_raw_dcs;

    // GSM 7-bits default alphabet, 8 bit data, and UCS2 are all supported.
    // Currently compression is not supported, so if compression is specified,
    // user data will be transmitted transparently as 8 bit data
    BYTE                raw_dcs;    // see GSM 03.38/4
    sms1_decoded_dcs_t  decoded_dcs;
} sms1_tx_dcs_t;

typedef struct
{
    // see GSM 03.40/9.2.3.24.1 and 9.2.3.24.8

    // Concatenated short message reference number. For 8 bits reference number,
    // use only the lower octet.
    WORD            concatenated_sm_ref_number;

    // Maximum number of short messages in the concatenated short message.
    // The value shall start at 1 and remain constant for every short message
    // which makes up the concatenated short message.
    BYTE            max_no_sm_in_concat_sm;

    // Sequence number of the current short message.
    // The value shall start at 1 and increment by one for every
    // short message sent within the concatenated short message.
    BYTE            sequence_no_current_sm;
} sms1_concat_sm_t;

typedef enum
{
    // this enum is composed of bit fields
    SMS1_UDH_USED_NONE              = 0x0000,
    SMS1_UDH_USED_8_BIT_CONCAT_SM   = 0x0001,
    SMS1_UDH_USED_16_BIT_CONCAT_SM  = 0x0002,
} sms1_udh_used_t;

typedef struct
{
    // The only headers currently supported are the
    // 8 and 16 bits concatenated short message. This field is set to
    // SMS1_UDH_USED_NONE if no headers are used.
    sms1_udh_used_t     udh_used;

    // Used only if a concatenated short message header was reported in
    // the field above.
    sms1_concat_sm_t    concat_sm;

    // This field contains the actual number of octets in the next field. If this
    // field is zero, then the next field is not used. For 7-bit and 8-bit data
    // this field will be the number of characters. For UCS2 data, this field
    // will be twice the number of characters, as each character occupies two
    // octets.
    UINT                data_length;

    // User data in this message is stripped of headers, and it is decoded if
    // not compressed and 7-bit default alphabet is used.
    char                data[SMS1_MAX_DATA_LENGTH];
} sms1_rx_user_data_t;

// Transmit and receive are handled with the same data structure.
// Use different names to make the code readable.

typedef sms1_rx_user_data_t sms1_tx_user_data_t;

typedef enum
{
    SMS1_VALIDITY_PERIOD_UNUSED,
    SMS1_VALIDITY_PERIOD_ENHANCED,  // enhanced mode, currently not supported
    SMS1_VALIDITY_PERIOD_RELATIVE,
    SMS1_VALIDITY_PERIOD_ABSOLUTE,
} sms1_validity_period_format_t;

typedef struct
{
    // Enhanced validity period is currently not supported.
    sms1_validity_period_format_t   format;

    // Period of time in minutes that the message should be valid. Maximum is 635,040
    // minutes, which corresponds to 63 weeks. Higher values will
    // be set to the maximum. Values lower than 5 minutes will be set to 5.
    // This field is used only if the format above is selected as relative.
    UINT                            relative;

    // When the message should expire. This field is used only if the format
    // above is selected as absolute.
    sms1_time_t                     absolute;

} sms1_validity_period_t;


///////////////////////////////////////////////////////////////////////////////
// TRANSMITTED MESSAGE DEFINITIONS
///////////////////////////////////////////////////////////////////////////////


typedef struct
{
    // for usage of these fields see GSM 03.40/9.2.2.2
    BOOL                    reject_duplicates;
    BOOL                    reply_path;
    BOOL                    status_report_request;
    BYTE                    message_reference;
    sms1_address_t          destination_address;
    sms1_pid_t              protocol_identifier;    // used only by application
    sms1_tx_dcs_t           tx_dcs;
    sms1_validity_period_t  validity_period;

    // Alphabets currently supported are default 7-bit GSM alphabet, 8-bit data,
    // and UCS2. If the default 7-bit GSM alphabet is used, then each 7-bit
    // character should occupy one octet, it will be encoded by layer 3.
    // Otherwise data will be passed transparently.

    // Maximum data length depends on several factors. For example, if the
    // default GSM 7-bits alphabet is used, maximum would be 160 octets, but if
    // 8-bit concatenated header is used, maximum is 153. It will be 140 octets
    // for 8-bit data or UCS2 encoding. Headers are not included in this length.
    // If data_length is zero, then data field is not used.
    sms1_tx_user_data_t     user_data;
} sms1_tx_submit_pdu_t;

typedef struct
{
    // for usage of these fields see GSM 03.40/9.2.2.1
    BOOL                more_messages_to_send;  // TRUE if more messages
    BOOL                reply_path;
    BOOL                status_report_indication;
    sms1_address_t      originating_address;
    sms1_pid_t          protocol_identifier;    // used only by application
    sms1_tx_dcs_t       tx_dcs;

    // This field should be the same as received by the transfer layer
    // when a message was submitted.
    sms1_time_t         service_centre_time_stamp;

    // Alphabets currently supported are default 7-bit GSM alphabet, 8-bit data,
    // and UCS2. If default 7-bit GSM alphabet is used, then each 7-bit character
    // should occupy one octet, it will be encoded by layer 3. Otherwise data
    // will be passed transparently.

    // Maximum data length depends on several factors. For example, if default
    // GSM 7-bits alphabet is used, maximum would be 160 octets, but if 8-bit
    // concatenated header is used, maximum is 153. It will be 140 octets for
    // 8-bit data or UCS2 encoding. Headers are not included in this length.
    // If data_length is zero, then data field is not used.
    sms1_tx_user_data_t user_data;
} sms1_tx_deliver_pdu_t;

typedef struct
{
    // for usage of these fields see GSM 03.40/9.2.2.4

    BOOL                status_report_request;
    BYTE                message_reference;
    sms1_pid_t          protocol_identifier;    // used only by application
    sms1_command_type_t command_type;
    BYTE                message_number;

    // Set address value to the empty string for TP-Command-Types that do not
    // operate on a specific SM.
    sms1_address_t      destination_address;

    // The TP-Command-Data field contains data relating to the operation
    // requested by the MS, which is to be performed at the SC. The maximum
    // length of this field is 157 octets. The usage and provision of the
    // optional TP-Command-Data field will be determined by the function
    // selected by the TP-Command-Type field. (GSM 03.40/9.2.3.21)

    // There is no possibility to specify encoding for the command PDU. Therefore,
    // data is assumed to be encoded as 8-bits, uncompressed, because
    // TP-UDL is defined as the number of septets or octets
    // according to the encoding type, whereas TP-UDL is defined as the number of
    // octets contained in the data field. By symetry this implies 8-bits data.
    // See GSM 03.40/9.2.3.20 and 9.2.3.16.

    // The maximum guaranteed length of TP-CD is 146 octets. In order to
    // achieve the maximum stated above (156 octets), the TP-DA field must
    // have a length of 2 octets. (Note at end of GSM 03.40/9.2.2.4).

    // It is not recommended to use more than 146 octets. Headers are not
    // included in this length. If data_length is zero, then the data field
    // in this structure is not used (but there could be headers present).
    sms1_tx_user_data_t user_data;
} sms1_tx_command_pdu_t;

typedef struct // used for transmitted status reports
{
    // for usage of these fields see GSM 03.40/9.2.2.3
    BOOL                more_messages_to_send;      // TRUE if more messages
    sms1_srq_t          status_report_qualifier;
    BYTE                message_reference;
    sms1_address_t      recipient_address;
    sms1_time_t         service_centre_time_stamp;
    sms1_time_t         discharge_time;
    sms1_status_t       status;
    BOOL                use_protocol_identifier;    // next field used if TRUE
    sms1_pid_t          protocol_identifier;        // used only by application

    // If use_data_coding_scheme is TRUE, then encoding is handled according to
    // the value of tx_dcs. By default the encoding is 7-bit GSM alphabet.
    BOOL                use_data_coding_scheme;
    sms1_tx_dcs_t       tx_dcs;

    // Alphabets currently supported are default 7-bit GSM alphabet, 8-bit data,
    // and UCS2. If default 7-bit GSM alphabet is used, then each 7-bit character
    // should occupy one octet, it will be encoded by layer 3. Otherwise data
    // will be passed transparently.

    // Maximum data length depends on several factors. For example, if default
    // GSM 7-bits alphabet is used, maximum would be 163 octets, but if 8-bit
    // concatenated header is used, maximum is 156. It will be 143 octets for
    // 8-bit data or 142 for UCS2 encoding. Headers are not included in this
    // length. If data_length is zero, then data field is not used.
    sms1_tx_user_data_t user_data;
} sms1_tx_stat_rep_pdu_t;

// used for transmitted submit and deliver reports, positive and negative
typedef struct
{
    // for usage of these fields see GSM 03.40/9.2.2.1a and 9.2.2.2a

    // used only for negative reports (submit and deliver)
    sms1_failure_cause_t    failure_cause;

    // used only for submit reports (positive and negative)
    sms1_time_t         service_centre_time_stamp;

    BOOL                use_protocol_identifier;    // next field used if TRUE
    sms1_pid_t          protocol_identifier;        // used only by application

    // If use_data_coding_scheme is TRUE, then encoding is handled according to
    // the value of tx_dcs. By default the encoding is 7-bit GSM alphabet.
    BOOL                use_data_coding_scheme;
    sms1_tx_dcs_t       tx_dcs;

    // Alphabets currently supported are default 7-bit GSM alphabet, 8-bit data,
    // and UCS2. If default 7-bit GSM alphabet is used, then each 7-bit character
    // should occupy one octet, it will be encoded by layer 3. Otherwise data
    // will be passed transparently.

    // Maximum data length depends on several factors. For example, if default
    // GSM 7-bits alphabet is used, maximum would be 172 octets, but if 8-bit
    // concatenated header is used, maximum is 165. It will be 151 octets for
    // 8-bit data or 150 for UCS2 encoding. Headers are not included in this
    // length. The above is for a negative submit report, it can be a few octets
    // more for other types of report. If data_length is zero, then data field is
    // not used.
    sms1_tx_user_data_t user_data;
} sms1_tx_report_pdu_t;


///////////////////////////////////////////////////////////////////////////////
// RECEIVED MESSAGE DEFINITIONS
///////////////////////////////////////////////////////////////////////////////


typedef struct
{
    sms_error_t             error;
    sms1_time_t             service_centre_time_stamp;

    // for usage of these fields see GSM 03.40/9.2.2.2
    BOOL                    reject_duplicates;
    BOOL                    reply_path;
    BOOL                    status_report_request;
    BYTE                    message_reference;
    sms1_address_t          destination_address;
    sms1_pid_t              protocol_identifier;    // used only by application

    // Layer 3 always presents the DCS in both raw and decoded formats
    // to the application.
    sms1_decoded_dcs_t      decoded_dcs;
    BYTE                    raw_dcs;                // see GSM 03.38/4
    sms1_validity_period_t  validity_period;
    sms1_rx_user_data_t     user_data;
} sms1_rx_submit_pdu_t;

typedef struct
{
    sms_error_t         error;

    // for usage of these fields see GSM 03.40/9.2.2.1
    BOOL                more_messages_to_send;  // TRUE if more messages
    BOOL                reply_path;
    BOOL                status_report_indication;
    sms1_address_t      originating_address;
    sms1_pid_t          protocol_identifier;    // used only by application
    // Layer 3 always presents the DCS in both raw and decoded formats
    // to the application.
    sms1_decoded_dcs_t  decoded_dcs;
    BYTE                raw_dcs;                // see GSM 03.38/4
    sms1_time_t         service_centre_time_stamp;
    sms1_rx_user_data_t user_data;
} sms1_rx_deliver_pdu_t;

typedef struct
{
    sms_error_t         error;

    // for usage of these fields see GSM 03.40/9.2.2.4

    BOOL                status_report_request;
    BYTE                message_reference;
    sms1_pid_t          protocol_identifier;    // used only by application
    sms1_command_type_t command_type;
    BYTE                message_number;
    sms1_address_t      destination_address;

    // Next field contains command data, which is assumed to be 8-bits encoded.
    sms1_rx_user_data_t user_data;
} sms1_rx_command_pdu_t;

typedef struct // used for transmitted status reports
{
    sms_error_t         error;

    // for usage of these fields see GSM 03.40/9.2.2.3
    BOOL                more_messages_to_send;      // TRUE if more messages
    sms1_srq_t          status_report_qualifier;
    BYTE                message_reference;
    sms1_address_t      recipient_address;
    sms1_time_t         service_centre_time_stamp;
    sms1_time_t         discharge_time;
    sms1_status_t       status;

    // protocol identifier received
    BOOL                pid_received;   // next field used if this field is TRUE
    sms1_pid_t          protocol_identifier;    // used only by application

    // data coding scheme received
    // If this field is TRUE, then the next two fields are used; if DCS not received,
    // user data is assumed to be encoded according to the default GSM 7-bit
    // alphabet.
    BOOL                dcs_received;
    sms1_decoded_dcs_t  decoded_dcs;
    BYTE                raw_dcs;                // see GSM 03.38/4

    sms1_rx_user_data_t user_data;
} sms1_rx_stat_rep_pdu_t;


// used for received submit and deliver reports, positive and negative
typedef struct
{
    sms_error_t         error;

    // for usage of these fields see GSM 03.40/9.2.2.1a and 9.2.2.2a

    // used only for negative reports (submit and deliver)
    sms1_failure_cause_t    failure_cause;

    // used only for submit reports (positive and negative)
    sms1_time_t         service_centre_time_stamp;

    // protocol identifier received
    BOOL                pid_received;   // next field used if this field is TRUE
    sms1_pid_t          protocol_identifier;    // used only by application

    // data coding scheme received
    // If dcs_received is TRUE, then encoding is handled according to
    // the value of decoded_dcs. By default the encoding is 7-bit GSM alphabet.
    BOOL                dcs_received;
    sms1_decoded_dcs_t  decoded_dcs;
    BYTE                raw_dcs;                // see GSM 03.38/4

    sms1_rx_user_data_t user_data;
} sms1_rx_report_pdu_t;


///////////////////////////////////////////////////////////////////////////////
// MAIN MESSAGE DEFINITIONS
///////////////////////////////////////////////////////////////////////////////


// Message sent between application and layer 3 (both ways) for protocol 1.
typedef struct
{
    // application message type (some messages carry PDUs, some don't, see below)
    sms1_app_msg_type_t             app_msg_type;

    // PDU type - indicates which member of the union below is used.
    // Must be set to SMS1_PDU_TYPE_NOT_USED if pdu field none used.
    // Messages of the following types do not carry PDUs:
    //  *SMS1_CM_REL_IND, SMS1_APP_SMS_REJ, SMS1_APP_DL_ERR (layer 3 -> application)
    //  *SMS1_TL_EST_IND, SMS1_TL_SMS_REL                   (application -> layer 3)
    sms1_pdu_type_t                 pdu_type;

    union
    {
        // received PDUs
        sms1_rx_submit_pdu_t        rx_sub;
        sms1_rx_deliver_pdu_t       rx_dlv;
        sms1_rx_command_pdu_t       rx_cmd;
        sms1_rx_stat_rep_pdu_t      rx_sr;  // status report
        sms1_rx_report_pdu_t        rx_rep; // submit and deliver reports

        // sent PDUs
        sms1_tx_submit_pdu_t        tx_sub;
        sms1_tx_deliver_pdu_t       tx_dlv;
        sms1_tx_command_pdu_t       tx_cmd;
        sms1_tx_stat_rep_pdu_t      tx_sr;  // status report
        sms1_tx_report_pdu_t        tx_rep; // submit and deliver reports
    } pdu;

    // layer 2 error - applicable only if the aplication message type is
    // SMS1_APP_SMS_REJ or SMS1_APP_DL_ERR
    sms1_tl_data_rej_cause_t        tl_data_rej_cause;
} sms1_app_message_t;


///////////////////////////////////////////////////////////////////////////////
// CONFIGURATION PARAMETERS
///////////////////////////////////////////////////////////////////////////////


typedef enum    // application type
{
    SMS1_APP_TYPE_RESERVED, // should not be used
    SMS1_APP_TYPE_CLIENT,   // equivalent to mobile station in GSM 03.40
    SMS1_APP_TYPE_SC,       // service center
} sms1_app_type_t;

typedef struct
{
    sms1_app_type_t sms1_app_type;  // application type

    HANDLE          app_h;          // application handle as returned by SDK

    // This should be set by the application to be twice the number of BRI
    // ports, as each port can have up to two B-channels, or 30 times the
    // number of PRI ports (if the the PRI card supports 30 channels).
    int             max_number_calls;

    // Maximum number of events for layer 2 and 3. There is no fixed rule
    // for this number, it depends on load, but here is a suggestion:
    // Max. Number of calls         Number of events
    //          2                           10
    //          8                           25
    //          30                          60
    //          >200                        300
    int             max_number_l2_events;
    int             max_number_l3_events;

    // minimum time before first frame, must be multiple of 100 ms,
    // minimum value is 100 ms (T10 in ES 201 912/5.3.1)
    int             t10_ms;

    // maximum time before first frame (ES 201 912/5.3.3.3)
    int             t5333_ms;

    // minimum time between two frames, minimum value is 100 ms
    // (T11 in ES 201 912/5.3.1)
    int             t11_ms;
} sms1_parameters_t;


///////////////////////////////////////////////////////////////////////////////
// GLOBAL FUNCTIONS EXPORTED BY THE SDK
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
extern "C" {
#endif

// start and stop SMS protocol 1
DIVA_SERVER_API sms_error_t DivaSms1Start (sms1_parameters_t *sms1_parameters);
DIVA_SERVER_API void DivaSms1Stop(void);

// create a new call and delete it
DIVA_SERVER_API sms_error_t DivaSms1NewCall(HANDLE hcall, BOOL call_received);
DIVA_SERVER_API sms_error_t DivaSms1DeleteCall(HANDLE hcall);

// send message to layer 3
DIVA_SERVER_API sms_error_t DivaSms1SendMsgToL3(HANDLE hcall,
                                                sms1_app_message_t *message);

// This function should be called by the application after reading the message.
// If it fails to do so, the output buffer will overflow and messages will
// be lost.
DIVA_SERVER_API sms_error_t DivaSms1ReleaseMsgReceived(HANDLE hcall,
                                       sms1_app_message_t *sms1_app_message);

// Returns the local time in a format suitable for use by the SMS SDK. This
// function can be used for example in Service Center applications to obtain
// the discharge time, necessary for status reports PDUs.
DIVA_SERVER_API void DivaSms1GetLocalTime(sms1_time_t *local_time);

#ifdef __cplusplus
}
#endif

#endif /* _SMSSDK_H_ */
